//                    -*-C-*-

//  This file is part of Vgluefftw.
//  Copyright 2006,2007 Ampersand Inc., Doug Campbell
//
//  Vgluefftw is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  Vgluefftw is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Vgluefftw; if not, see <http://www.gnu.org/licenses/>.

/*  This is C code, passed unchanged to the .c file  */
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include <fftw3.h>

unsigned char linear2ulaw (int16_t sample)
{
  static unsigned char exp_lut[256] = {0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
				       4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
				       5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
				       5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
				       6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				       6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				       6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				       6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
				       7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
  unsigned char sign, exponent, mantissa, ulawbyte;
  int doing_zerotrap = 0;
  int16_t bias = 132;             /* add-in bias for 16 bit samples */
  int16_t clip = 32767 - bias;

  /* Get the sample into sign-magnitude. */
  sign = (sample >> 8) & 0x80;		/* set aside the sign */
  if (sign != 0) {sample = -sample;};		/* get magnitude */
  if (sample > clip) {sample = clip;};		/* clip the magnitude */

  /* Convert from 16 bit linear to ulaw. */
  sample = sample + bias;
  exponent = exp_lut[(sample >> 7) & 0xFF];
  mantissa = (sample >> (exponent + 3)) & 0x0F;
  ulawbyte = ~(sign | (exponent << 4) | mantissa);
  if (doing_zerotrap && (ulawbyte == 0)) {ulawbyte = 0x02;};
  return(ulawbyte);
}

int16_t ulaw2linear(unsigned char ulawbyte)
{
  static int16_t exp_lut[8] = {0,132,396,924,1980,4092,8316,16764};
  unsigned char sign, exponent;
  int16_t mantissa;
  int16_t sample;

  ulawbyte = ~ulawbyte;
  sign = (ulawbyte & 0x80);
  exponent = (ulawbyte >> 4) & 0x07;
  mantissa = ulawbyte & 0x0F;
  sample = exp_lut[exponent] + (mantissa << (exponent + 3));
  if (sign != 0) {sample = -sample;};

  return(sample);
}


/*  This starts the xsubpp-translated code  */

MODULE = Vgluefftw   PACKAGE = Vgluefftw

void
_PerlFree (sv)
        SV *sv
INIT:
    void *ptr;
PPCODE:
    ptr = (void *) SvIV(SvRV(sv));
    if (ptr != NULL)
    {
	Safefree (ptr);
    };

MODULE = Vgluefftw   PACKAGE = Vgluefftw

SV *
_fft_packet (int samples, int bytes_per_sample, char *packet, double mag_threshhold, double noise_threshhold)
INIT:
    fftw_complex *in;
    fftw_complex *out;
    double average, max, magnitude;
    double tone_threshhold, tone_freqs[2], tone_mags[2];
    double frequency;
    int tone_count, i;
    fftw_plan p;
PPCODE:
 /*  Given a packet containing samples samples (signed) of bytes_per_sample  */
 /*  (must be either 1, 2, or 4) replaces the first half of the values       */
 /*  with the fft frequency magnitudes of the packet.                        */
 /*  (bytes_per_sample of 1 assumes input is mulaw)                          */
 /*  mag_threshhold is 1.0 or greater, the multiplier greater than the       */
 /*  average frequency that the tone frequency must be to be considered      */
 /*  a tone.                                                                 */
 /*  noise_threshhold is an absolute magnitude below which anything will be  */
 /*  considered irrelevant (noise).                                          */
 /*  Returns:  (tone_freq, avg_mag, max_mag, threshhold, hits)               */
 /*    tone_freq = index of tone freq found (cycles per packet) or 0 if none */
 /*                (can be number halfway between two indexes)               */
 /*    avg_mag = average magnitude of tone freqs                             */
 /*    max_mag = maximum magnitude of tone freqs                             */
 /*    threshhold = computed absolute threshhold                             */
 /*    hits = number of tone frequencies that exceeded threshhold            */
 /*  And the first half of the packet's values contain the FFT magnitudes    */

    //  Set up in and out buffers
    in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * samples);
    out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * samples);
    p = fftw_plan_dft_1d(samples, in, out, FFTW_FORWARD, FFTW_ESTIMATE);
    for (i = 0; i < samples; ++i)
    {
	fftw_complex *destination = & in[i];
	(*destination)[1] = 0;
	if (bytes_per_sample == 1)
	{
	    (*destination)[0] =
		(double) ulaw2linear (((unsigned char *)packet)[i]);
	    // printf ("sample[%d] = %g\n", i, (*destination)[0]);
	}
	else if (bytes_per_sample == 2)
	{
	    (*destination)[0] = (double) (((int16_t *) packet)[i]);
	}
	else
	{
	    (*destination[0]) = (double) (((int32_t *) packet)[i]);
	};
    };

    //  perform the FFT
    fftw_execute(p);

    //  copy out FFT results
    average = 0;
    max = 0;
    for (i = 0; i < (samples/2); ++i)
    {
	magnitude = sqrt (out[i][0] * out[i][0] +
			  out[i][1] * out[i][1]) / samples;
	if (bytes_per_sample == 1)
	{
	    //  Down-normalize to byte from 16-bit linear
	    magnitude /= 256;
	};
	// printf ("magnitude[%d] = %g\n", i, magnitude);
	average += magnitude;
	if (magnitude > max) {max = magnitude;};
	if (bytes_per_sample == 1)
	{
	    (((unsigned char *)packet)[i]) = (unsigned char) magnitude;
	}
	else if (bytes_per_sample == 2)
	{
	    (((int16_t *) packet)[i]) = (int16_t) magnitude;
	}
	else
	{
	    (((int32_t *) packet)[i]) = (int32_t) magnitude;
	};
    };
    average /= samples;

    //  Clean up the FFTW allocations
    fftw_destroy_plan(p);
    fftw_free(in);
    fftw_free(out);

    //  Compute threshhold
    tone_threshhold = average * mag_threshhold;
    if (tone_threshhold < noise_threshhold * mag_threshhold)
    {
	tone_threshhold = noise_threshhold * mag_threshhold;
    };

    //  Look for tone
    tone_count = 0;
    if ((max > noise_threshhold) && (max > tone_threshhold))
    {
	for (i = 0; i < (samples/2); ++i)
	{
	    if (bytes_per_sample == 1)
	    {
		magnitude = (double) (((char *)packet)[i]);
	    }
	    else if (bytes_per_sample == 2)
	    {
		magnitude = (double) (((int16_t *) packet)[i]);
	    }
	    else
	    {
		magnitude = (double) (((int32_t *) packet)[i]);
	    };
	    if (magnitude >= tone_threshhold)
	    {
		if (tone_count < 2)
		{
		    tone_mags[tone_count] = magnitude;
		    tone_freqs[tone_count] = (double) i;
		};
		++tone_count;
	    };
	};
    };
    frequency = 0;
    if ((tone_count < 3) && (tone_count > 0))
    {
	if (tone_count == 1)
	{
	    //  Found it - single frequency
	    frequency = tone_freqs[0];
	}
	else if (tone_freqs[0] == tone_freqs[1] - 1)
	{
	    //  Found it - adjacent frequencies
	    frequency = (tone_freqs[0] + tone_freqs[1]) / 2;
	};
    };

    EXTEND (SP, 5);
    XPUSHs (sv_2mortal (newSVnv(frequency)));
    XPUSHs (sv_2mortal (newSVnv(average)));
    XPUSHs (sv_2mortal (newSVnv(max)));
    XPUSHs (sv_2mortal (newSVnv(tone_threshhold)));
    XPUSHs (sv_2mortal (newSViv(tone_count)));


MODULE = Vgluefftw   PACKAGE = Vgluefftw

SV *
_ulaw_to_linear (int byte)
INIT:
    int16_t result;
PPCODE:
    result = ulaw2linear (byte);

    EXTEND (SP, 1);
    XPUSHs (sv_2mortal (newSViv(result)));


MODULE = Vgluefftw   PACKAGE = Vgluefftw

SV *
_linear_to_ulaw (int sample)
INIT:
    unsigned char result;
PPCODE:
    result = linear2ulaw ((int16_t) sample);

    EXTEND (SP, 1);
    XPUSHs (sv_2mortal (newSViv((unsigned int) result)));
