#!perl -T --            -*-CPerl-*-

#!perl --            -*-CPerl-*-
#use blib;
#chdir ("/home/soup/doc/projects/voiceglue/dist/trunk/Vgluefftw");
#$::Verbose = 1;


##  Copyright 2006,2007 Doug Campbell
##
##  This file is part of Vgluefftw.
##
##  Vgluefftw is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 3 of the License, or
##  (at your option) any later version.
##
##  Vgluefftw is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with Vgluefftw; if not, see <http://www.gnu.org/licenses/>.

use Test::More tests => 14;
use Vgluefftw;

use strict;

sub check_ulaw
{
    my ($i, $s);
    for ($i = 0; $i < 255; ++$i)
    {
	$s = Vgluefftw::_ulaw_to_linear ($i);
	print STDOUT ($i, " -> ", $s,
		      " -> ", Vgluefftw::_linear_to_ulaw ($s), "\n");
    };
};

##  report_results ($bytes_per, $samples, $data, $freq, $avg, $max, $thresh, $hits, $start)
sub report_results
{
    my ($bytes_per) = shift (@_);
    my ($samples) = shift (@_);
    my ($data) = shift (@_);
    my ($freq) = shift (@_);
    my ($avg) = shift (@_);
    my ($max) = shift (@_);
    my ($thresh) = shift (@_);
    my ($hits) = shift (@_);
    my ($start) = shift (@_);
    my ($i, @mags);

    $::Verbose || return;

    defined ($start) || ($start = 0);

    if ($bytes_per == 1)
    {
	@mags = unpack ("C" . int ($samples / 2), $data);
    }
    else
    {
	@mags = unpack ("s" . int ($samples / 2), $data);
    };
    if ($start) {print STDOUT ("* ");};
    print STDOUT ("freq = " . $freq . ", ");
    print STDOUT ("avg = " . $avg . ", ");
    print STDOUT ("max = " . $max . ", ");
    print STDOUT ("thresh = " . $thresh . ", ");
    print STDOUT ("hits = " . $hits . "\n");
    for ($i = 0; $i < scalar (@mags); ++$i)
    {
	print STDOUT ("$i=" . $mags[$i]);
	print STDOUT (" ");
    };
    print STDOUT ("\n");
};

sub report_file_results
{
    my ($raw_results) = shift (@_);
    my ($packet_results);

    $::Verbose || return;

    foreach $packet_results (@$raw_results)
    {
	report_results (1, 160,
			$packet_results->{"mags"},
			$packet_results->{"freq"},
			$packet_results->{"avg"},
			$packet_results->{"max"},
			$packet_results->{"thresh"},
			$packet_results->{"hits"},
			$packet_results->{"found"});
    };
};

sub main
{
    my ($data1, $data2, $data3, $data4, $data5, $data6, $data7, $data8);
    my ($freq, $avg, $max, $thresh, $hits);
    my ($ok, $msg, $tones, $raw_results);
    my ($mag_thresh_2byte) = 2.0;
    my ($mag_thresh_1byte) = 2.0;
    my ($noise_thresh_2byte) = 30;
    my ($noise_thresh_1byte) = 5;
    my ($count_thresh) = 10;

    ##  16 samples, 1 cycle per sample
    $data1 = pack ("s16",
		   0,  3830,  7070,  9240,  10000,  9240,  7070,  3830,
		   0, -3830, -7070, -9240, -10000, -9240, -7070, -3830);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (16, 2, $data1, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 1, "data1: 16 samples, 1 cycle per sample");
    report_results (2, 16, $data1, $freq, $avg, $max, $thresh, $hits);

    ##  16 samples, 2 cycles per sample
    $data2 = pack ("s16",
		   0,  7070,  10000,  7070,  0, -7070, -10000, -7070,
		   0,  7070,  10000,  7070,  0, -7070, -10000, -7070);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (16, 2, $data2, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 2, "data1: 16 samples, 2 cycle per sample");
    report_results (2, 16, $data2, $freq, $avg, $max, $thresh, $hits);

    ##  16 samples, 3 cycles per sample (67.5 degrees)
    $data5 = pack ("s16",
		   0,  9240,  7070, -3830, -10000, -3830,  7070,  9240,
		   0, -9240, -7070,  3930,  10000,  3830, -7070, -9240);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (16, 2, $data5, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 3, "data1: 16 samples, 3 cycle per sample");
    report_results (2, 16, $data5, $freq, $avg, $max, $thresh, $hits);

    ##  16 samples, 4 cycles per sample
    $data3 = pack ("s16",
		   0,  10000,  0, -10000, 0,  10000,  0, -10000,
		   0,  10000,  0, -10000, 0,  10000,  0, -10000);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (16, 2, $data3, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 4, "data1: 16 samples, 4 cycle per sample");
    report_results (2, 16, $data3, $freq, $avg, $max, $thresh, $hits);

    ##  16 samples, 4.5 cycles per sample
    $data6 = pack ("s16",
		   0,  9807, -3826, -8314, 7071, 5555, -9238, -1950,
		   10000, -1950, -9238, 5555, 7071, -8314, -3826, 9807);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (16, 2, $data6, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 4.5, "data1: 16 samples, 4.5 cycle per sample");
    report_results (2, 16, $data6, $freq, $avg, $max, $thresh, $hits);

    ##  64 samples, 4 cycles per sample
    $data4 = pack ("s64",
		   0,  3830,  7070,  9240,  10000,  9240,  7070,  3830,
		   0, -3830, -7070, -9240, -10000, -9240, -7070, -3830,
		   0,  3830,  7070,  9240,  10000,  9240,  7070,  3830,
		   0, -3830, -7070, -9240, -10000, -9240, -7070, -3830,
		   0,  3830,  7070,  9240,  10000,  9240,  7070,  3830,
		   0, -3830, -7070, -9240, -10000, -9240, -7070, -3830,
		   0,  3830,  7070,  9240,  10000,  9240,  7070,  3830,
		   0, -3830, -7070, -9240, -10000, -9240, -7070, -3830);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (64, 2, $data4, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 4, "data1: 64 samples, 4 cycle per sample");
    report_results (2, 64, $data4, $freq, $avg, $max, $thresh, $hits);

    ##  16 samples, noise
    $data7 = pack ("s16",
		   -8453, 9301, 8956, -6913, -8916, 3204, 1008, 7698,
		   8343, -5839, -7701, -7347, 1026, -6601, 4504, 5738);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (16, 2, $data7, $mag_thresh_2byte, $noise_thresh_2byte);
    is ($freq, 0, "data1: 16 samples, noise");
    report_results (2, 16, $data7, $freq, $avg, $max, $thresh, $hits);

    ##  ulaw packet, 2 cycles
    $data8 = pack
      ("C160",
       0x7f, 0xbe, 0xaf, 0xa7, 0x9f, 0x9c, 0x98, 0x94,
       0x91, 0x8f, 0x8d, 0x8c, 0x8b, 0x8a, 0x89, 0x88,
       0x87, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86,
       0x87, 0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8f,
       0x91, 0x94, 0x98, 0x9c, 0x9f, 0xa7, 0xaf, 0xbe,
       0xff, 0x3e, 0x2f, 0x27, 0x1f, 0x1c, 0x18, 0x14,
       0x11, 0x0f, 0x0d, 0x0c, 0x0b, 0x0a, 0x09, 0x08,
       0x07, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06,
       0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0f,
       0x11, 0x14, 0x18, 0x1c, 0x1f, 0x27, 0x2f, 0x3e,
       0xff, 0xbe, 0xaf, 0xa7, 0x9f, 0x9c, 0x98, 0x94,
       0x91, 0x8f, 0x8d, 0x8c, 0x8b, 0x8a, 0x89, 0x88,
       0x87, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86, 0x86,
       0x87, 0x88, 0x89, 0x8a, 0x8b, 0x8c, 0x8d, 0x8f,
       0x91, 0x94, 0x98, 0x9c, 0x9f, 0xa7, 0xaf, 0xbe,
       0xff, 0x3e, 0x2f, 0x27, 0x1f, 0x1c, 0x18, 0x14,
       0x11, 0x0f, 0x0d, 0x0c, 0x0b, 0x0a, 0x09, 0x08,
       0x07, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06, 0x06,
       0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0f,
       0x11, 0x14, 0x18, 0x1c, 0x1f, 0x27, 0x2f, 0x7e);
    ($freq, $avg, $max, $thresh, $hits) =
      Vgluefftw::_fft_packet
	  (160, 1, $data8, $mag_thresh_1byte, $noise_thresh_1byte);
    is ($freq, 2, "data8: ulaw packet 2 cycles");
    report_results (1, 160, $data8, $freq, $avg, $max, $thresh, $hits);

    ##  Full file tone detection
    ($ok, $msg, $tones, $raw_results) =
      Vgluefftw::find_tones
	  ("t/arpeggio.ulaw",
	   $mag_thresh_1byte, $noise_thresh_1byte, $count_thresh, 1);
    is ($msg, "", "result message from find_tones()");
    is ($ok, 1, "result code from find_tones()");
    is (join (",", @$tones), "400,500,600,800",
	"arpeggio tone list from find_tones()");
    report_file_results ($raw_results);

    ##  Harder full file case
    ($ok, $msg, $tones) =
      Vgluefftw::find_tones
	  ("t/400_not_200.ulaw",
	   $mag_thresh_1byte, $noise_thresh_1byte, $count_thresh);
    is ($msg, "", "result message from find_tones()");
    is ($ok, 1, "result code from find_tones()");
    is (join (",", @$tones), "400",
	"400_not_200 tone list from find_tones()");
    report_file_results ($raw_results);
};

main();
